@@ -39,69 +39,6 @@ struct WritebackStats {
     nr_write_errors: u64,
 }
 
-/// Updates the progress bar in the reporter as the stats are
-/// updated by the copier threads.
-struct ProgressReporter {
-    report: Arc<Report>,
-    inner: Mutex<WritebackStats>,
-}
-
-impl ProgressReporter {
-    /// nr_blocks - nr to be copied
-    fn new(report: Arc<Report>, nr_blocks: u64) -> Self {
-        Self {
-            report,
-            inner: Mutex::new(WritebackStats {
-                nr_blocks,
-                nr_copied: 0,
-                nr_read_errors: 0,
-                nr_write_errors: 0,
-            }),
-        }
-    }
-
-    /// Adds stats into the internal base.  Then updates the progress.
-    fn inc_stats(&self, stats: &CopyStats) {
-        let mut inner = self.inner.lock().unwrap();
-        inner.nr_copied += stats.nr_copied;
-        inner.nr_read_errors += stats.read_errors.len() as u64;
-        inner.nr_write_errors += stats.write_errors.len() as u64;
-
-        self.report.set_sub_title(&format!(
-            "read errors {}, write errors {}",
-            inner.nr_read_errors, inner.nr_write_errors
-        ));
-
-        let percent = (inner.nr_copied * 100)
-            .checked_div(inner.nr_blocks)
-            .unwrap_or(100);
-        self.report.progress(percent as u8);
-    }
-
-    /// Accessor for the current stats
-    fn _stats(&self) -> WritebackStats {
-        let inner = self.inner.lock().unwrap();
-        (*inner).clone()
-    }
-}
-
-impl CopyProgress for ProgressReporter {
-    // This doesn't update the data fields, that is left until the copy
-    // batch is complete.
-    fn update(&self, stats: &CopyStats) {
-        let inner = self.inner.lock().unwrap();
-
-        self.report.set_sub_title(&format!(
-            "read errors {}, write errors {}",
-            inner.nr_read_errors + stats.read_errors.len() as u64,
-            inner.nr_write_errors + stats.write_errors.len() as u64
-        ));
-
-        let percent = (inner.nr_copied + stats.nr_copied * 100) / inner.nr_blocks;
-        self.report.progress(percent as u8);
-    }
-}
-
 //-----------------------------------------
 
 // Indicates whether a block should be copied.  There are differences
@@ -201,7 +138,7 @@ fn count_mappings<F: Fn(&Mapping) -> bool>(
     sb: &Superblock,
     predicate: F,
 ) -> anyhow::Result<u32> {
-    let w = ArrayWalker::new(engine.clone(), true);
+    let w = ArrayWalker::new(engine.as_ref(), true);
     let v = MappingCounter::new(predicate);
     w.walk(&v, sb.mapping_root)?;
     Ok(v.count.load(Ordering::SeqCst) as u32)
@@ -214,7 +151,7 @@ fn v2_read_dirty_bitset(
     sb: &Superblock,
 ) -> anyhow::Result<(u32, RoaringBitmap)> {
     let (cbits, err) = read_bitset_checked(
-        engine,
+        engine.as_ref(),
         sb.dirty_root.unwrap(),
         sb.cache_blocks as usize,
         true,
@@ -330,7 +267,7 @@ struct BitsetUpdater {
 impl BitsetUpdater {
     fn new(engine: Arc<dyn IoEngine + Sync + Send>, root: u64) -> anyhow::Result<Self> {
         let mut path = Vec::new();
-        let bitmaps_map = btree_to_map::<u64>(&mut path, engine.clone(), true, root)?;
+        let bitmaps_map = btree_to_map::<u64>(&mut path, engine.as_ref(), true, root)?;
 
         let mut bitmaps = Vec::with_capacity(bitmaps_map.len());
         for (i, (k, v)) in bitmaps_map.iter().enumerate() {
@@ -484,7 +421,6 @@ impl ThreadedCopier {
             let stats = copier
                 .copy(&ops, progress.clone())
                 .map_err(|e| anyhow!("copy failed: {}", e))?;
-            progress.inc_stats(&stats);
 
             {
                 for op in stats.read_errors {
@@ -525,7 +461,7 @@ fn update_v1_metadata(
     cleaned_blocks: &RoaringBitmap,
 ) -> anyhow::Result<()> {
     let mut path = Vec::new();
-    let ablocks = btree_to_value_vec::<u64>(&mut path, ctx.engine.clone(), true, sb.mapping_root)?;
+    let ablocks = btree_to_value_vec::<u64>(&mut path, ctx.engine.as_ref(), true, sb.mapping_root)?;
     let max_entries = array::calc_max_entries::<Mapping>();
     path.clear();
 
@@ -646,7 +582,7 @@ fn copy_dirty_blocks(
         copy_all_dirty_blocks(ctx.engine.clone(), sb, copier, ctx.report.clone())?
     };
 
-    // Retry blocks ignoed by vectored io
+    // Retry blocks ignored by vectored io
     if !read_failed.is_empty() || !write_failed.is_empty() {
         let copier = Box::new(
             SyncCopier::<SimpleBlockIo<File>>::from_path(
@@ -715,7 +651,7 @@ fn copy_all_dirty_blocks(
 
     // Build batches of copy operations and pass them to the copy thread
     let batcher = WritebackBatcher::new(1_000_000, tx, selector);
-    let w = ArrayWalker::new(engine, true);
+    let w = ArrayWalker::new(engine.as_ref(), true);
 
     // FIXME: do something with this
     w.walk(&batcher, sb.mapping_root)?;
@@ -741,7 +677,7 @@ fn copy_selected_blocks(
 
     let mut batcher = CopyOpBatcher::new(1_000_000, tx);
 
-    let ablocks = btree_to_value_vec::<u64>(&mut vec![0], engine.clone(), true, sb.mapping_root)?;
+    let ablocks = btree_to_value_vec::<u64>(&mut vec![0], engine.as_ref(), true, sb.mapping_root)?;
     let entries_per_block = array::calc_max_entries::<Mapping>();
     let mut index = 0;
     let b = engine.read(ablocks[index])?;
