@@ -1,9 +1,12 @@
 pub mod base;
 pub mod batcher;
+pub mod report;
 pub mod rescue_copier;
 pub mod sync_copier;
+pub mod wrapper;
 
 pub use crate::copier::base::*;
+pub use crate::copier::report::*;
 pub use crate::copier::rescue_copier::RescueCopier;
 pub use crate::copier::sync_copier::SyncCopier;
 
