@@ -32,8 +32,8 @@ fn mk_random_ops(src: Range<u64>, dst: Range<u64>, nr_ops: usize) -> Vec<CopyOp>
     let mut src_blocks: Vec<u64> = src.into_iter().collect();
     let mut dst_blocks: Vec<u64> = dst.into_iter().collect();
 
-    src_blocks.shuffle(&mut rand::thread_rng());
-    dst_blocks.shuffle(&mut rand::thread_rng());
+    src_blocks.shuffle(&mut rand::rng());
+    dst_blocks.shuffle(&mut rand::rng());
 
     src_blocks
         .into_iter()
@@ -152,7 +152,7 @@ impl CopierTest {
             block_size: block_size as usize,
             nr_src_blocks: nr_src_blocks as u64,
             nr_dst_blocks: nr_dst_blocks as u64,
-            seed: rand::thread_rng().gen::<u64>(),
+            seed: rand::rng().random::<u64>(),
             faulty_src_blocks: RoaringBitmap::new(),
             faulty_dst_blocks: RoaringBitmap::new(),
         }
@@ -177,7 +177,7 @@ impl CopierTest {
         self.src.invalidate(bytes);
 
         for b in block_begin..block_end {
-            self.faulty_src_blocks.push(b);
+            let _ = self.faulty_src_blocks.try_push(b);
         }
 
         Ok(())
@@ -190,7 +190,7 @@ impl CopierTest {
         self.dst.invalidate(bytes);
 
         for b in block_begin..block_end {
-            self.faulty_dst_blocks.push(b);
+            let _ = self.faulty_dst_blocks.try_push(b);
         }
 
         Ok(())
