@@ -25,15 +25,15 @@ struct EraEmitter<'a> {
     emitter: Mutex<&'a mut dyn MetadataVisitor>,
 }
 
-impl<'a> EraEmitter<'a> {
-    pub fn new(emitter: &'a mut dyn MetadataVisitor) -> EraEmitter {
+impl EraEmitter<'_> {
+    pub fn new(emitter: &mut dyn MetadataVisitor) -> EraEmitter<'_> {
         EraEmitter {
             emitter: Mutex::new(emitter),
         }
     }
 }
 
-impl<'a> dump_utils::ArrayVisitor<u32> for EraEmitter<'a> {
+impl dump_utils::ArrayVisitor<u32> for EraEmitter<'_> {
     fn visit(&self, index: u64, b: ArrayBlock<u32>) -> anyhow::Result<()> {
         let begin = index as u32 * b.header.max_entries;
         let end = begin + b.header.nr_entries;
@@ -127,7 +127,7 @@ impl<'a> LogicalEraEmitter<'a> {
     }
 }
 
-impl<'a> dump_utils::ArrayVisitor<u32> for LogicalEraEmitter<'a> {
+impl dump_utils::ArrayVisitor<u32> for LogicalEraEmitter<'_> {
     fn visit(&self, index: u64, b: ArrayBlock<u32>) -> anyhow::Result<()> {
         let mut inner = self.inner.lock().unwrap();
 
@@ -162,7 +162,7 @@ impl<'a> OutputVisitor<'a> {
     }
 }
 
-impl<'a> MetadataVisitor for OutputVisitor<'a> {
+impl MetadataVisitor for OutputVisitor<'_> {
     fn superblock_b(&mut self, sb: &ir::Superblock) -> anyhow::Result<ir::Visit> {
         output_context(self.out.superblock_b(sb))
     }
@@ -230,7 +230,7 @@ fn dump_writeset(
     repair: bool,
 ) -> anyhow::Result<()> {
     // TODO: deal with broken writeset
-    let bits = read_bitset(engine.clone(), ws.root, ws.nr_bits as usize, repair)?;
+    let bits = read_bitset(engine.as_ref(), ws.root, ws.nr_bits as usize, repair)?;
 
     out.writeset_b(&ir::Writeset {
         era,
@@ -240,11 +240,13 @@ fn dump_writeset(
     // [begin, end) denotes the range of set bits.
     let mut begin: u32 = 0;
     let mut end: u32 = 0;
+    let bits_per_block = fixedbitset::Block::BITS;
+    let bits_shift = bits_per_block.trailing_zeros();
     for (index, entry) in bits.as_slice().iter().enumerate() {
         let mut n = *entry;
 
-        if n == u32::MAX {
-            end = std::cmp::min(end + 32, ws.nr_bits);
+        if n == fixedbitset::Block::MAX {
+            end = std::cmp::min(end + bits_per_block, ws.nr_bits);
             continue;
         }
 
@@ -269,7 +271,7 @@ fn dump_writeset(
         }
 
         // emit the range if it ends before the entry boundary
-        let endpos = ((index as u32) << 5) + 32;
+        let endpos = (index as u32 + 1) << bits_shift;
         if end < endpos {
             if end > begin {
                 let m = ir::MarkedBlocks {
@@ -302,7 +304,7 @@ fn dump_eras(
     root: u64,
     ignore_non_fatal: bool,
 ) -> Result<()> {
-    let ablocks = collect_array_blocks_with_path(engine.clone(), ignore_non_fatal, root)?;
+    let ablocks = collect_array_blocks_with_path(engine.as_ref(), ignore_non_fatal, root)?;
     let emitter = EraEmitter::new(out);
     walk_array_blocks(engine, ablocks, &emitter)
 }
@@ -347,7 +349,7 @@ fn get_writesets_ordered(
 ) -> Result<Vec<(u32, Writeset)>> {
     let mut path = vec![0];
     let mut writesets =
-        btree_to_map::<Writeset>(&mut path, engine.clone(), repair, sb.writeset_tree_root)?;
+        btree_to_map::<Writeset>(&mut path, engine.as_ref(), repair, sb.writeset_tree_root)?;
 
     if sb.current_writeset.root != 0 {
         if writesets.contains_key(&(sb.current_era as u64)) {
@@ -404,7 +406,7 @@ fn collate_writesets(
     let mut archive = new_era_archive(sb.nr_blocks, archived_begin, writesets.len() as u32);
 
     for (index, (_era, ws)) in writesets.iter().enumerate() {
-        let bitset = read_bitset(engine.clone(), ws.root, ws.nr_bits as usize, repair)?;
+        let bitset = read_bitset(engine.as_ref(), ws.root, ws.nr_bits as usize, repair)?;
         collate_writeset(index as u32, &bitset, archive.as_mut())?;
     }
 
@@ -418,7 +420,7 @@ fn dump_eras_logical(
     era_archive: &dyn Archive,
     ignore_non_fatal: bool,
 ) -> Result<()> {
-    let ablocks = collect_array_blocks_with_path(engine.clone(), ignore_non_fatal, root)?;
+    let ablocks = collect_array_blocks_with_path(engine.as_ref(), ignore_non_fatal, root)?;
     let emitter = LogicalEraEmitter::new(out, era_archive);
     walk_array_blocks(engine, ablocks, &emitter)
 }
